import 'dart:async';

import 'package:flutter_test/flutter_test.dart';
import 'package:harmony_log/src/event/event.dart';
import 'package:harmony_log/src/level/level.dart';
import 'package:harmony_log/src/plain/format/format.dart';

void main() {
  group('LogPlainFormat', () {
    group('json', () {
      group('format', () {
        test('complete', () {
          final formatter = LogPlainFormat.json();
          final time = DateTime.now();
          final event = LogEvent(
            id: 1000,
            time: time,
            tag: 'tag',
            level: LogLevel.warning,
            message: 'message',
            zone: Zone.current,
            error: 'error',
            stackTrace: StackTrace.empty,
            extra: 'extra',
          );
          final format = formatter.format(event);
          expect(format, hasLength(1));
          expect(
              format.first,
              equals(
                '{"id":1000,"time":"${time.toIso8601String()}",'
                '"tag":"tag","level":"warning","message":"message",'
                '"error":"error","stackTrace":"","extra":"extra"},',
              ));
        });

        test('partial', () {
          final formatter = LogPlainFormat.json();
          final time = DateTime.now();
          final event = LogEvent(
            id: 1000,
            time: time,
            level: LogLevel.warning,
            message: 'message',
            zone: Zone.current,
          );
          final format = formatter.format(event);
          expect(format, hasLength(1));
          expect(
              format.first,
              equals(
                '{"id":1000,"time":"${time.toIso8601String()}",'
                '"tag":null,"level":"warning","message":"message",'
                '"error":null,"stackTrace":null,"extra":null},',
              ));
        });
      });

      test('start', () {
        final formatter = LogPlainFormat.json();
        final x = formatter.start();
        expect(x, hasLength(1));
        expect(x.first, equals('['));
      });

      test('end', () {
        final formatter = LogPlainFormat.json();
        final x = formatter.end();
        expect(x, hasLength(1));
        expect(x.first, equals(']'));
      });
    });
  });
}
